# Generated by roxygen2: do not edit by hand

export(HPZone_convert_dates)
export(HPZone_make_valid)
export(HPZone_necessary_scope)
export(HPZone_request)
export(HPZone_request_paginated)
export(HPZone_request_query)
export(HPZone_request_raw)
export(HPZone_setup)
export(HPZone_store_credentials)
export(test_HPZone_token)
importFrom(dplyr,coalesce)
importFrom(httr2,oauth_client)
importFrom(httr2,oauth_flow_client_credentials)
importFrom(httr2,req_body_raw)
importFrom(httr2,req_headers)
importFrom(httr2,req_oauth_client_credentials)
importFrom(httr2,req_perform)
importFrom(httr2,request)
importFrom(jsonlite,fromJSON)
importFrom(keyring,key_get)
importFrom(keyring,key_list)
importFrom(keyring,key_set_with_value)
importFrom(lubridate,ymd_hms)
importFrom(magrittr,"%>%")
importFrom(readxl,read_xlsx)
importFrom(rstudioapi,askForPassword)
importFrom(safer,decrypt_string)
importFrom(safer,encrypt_string)
importFrom(stats,setNames)
importFrom(stringr,fixed)
importFrom(stringr,regex)
importFrom(stringr,str_c)
importFrom(stringr,str_detect)
importFrom(stringr,str_locate)
importFrom(stringr,str_sub)
importFrom(stringr,str_to_lower)
